import os
import time

from datetime import datetime

class Controller():
    def __init__(self) -> None:
        self.startTime = time.time()

    def printHeader(self, path, hash, args):
        print("###########################################################################################")
        print("")
        print("ntrprtr by 5f0")
        print("Interpret bytes through different customizable actions")
        print("")
        print("                Arguments: ")
        print(''.join(f'                           {k}={v}\n' for k, v in vars(args).items()))
        print("Current working directory: " + os.getcwd())
        print("        Investigated File: " + path)
        print("")
        print("                      MD5: " + hash.md5)
        print("                   SHA256: " + hash.sha256)
        print("")        
        print("                 Datetime: " + datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
        print("")
        print("###########################################################################################")

    def printConfigSuccessMessage(self, name):
        print("")
        print("Template successfully created: " + name)
        print("")

    def printConfigFailureMessage(self, name):
        print("")
        print("Configuration File exists already: " + name)
        print("")
        self.printExecutionTime()

    def printTestfileSuccessMessage(self, name):
        print("")
        print("Testfile created successfully: " + name)
        print("")
        self.printExecutionTime()    

    def printExecutionTime(self):
        end = time.time()
        print("###########################################################################################")
        print("")
        print("Execution Time: " + str(end-self.startTime)[0:8] + " sec")
        print("")


from ntrprtr.action.ActionBase import ActionBase

class AsciiAction(ActionBase):
    def __init__(self):
        super().__init__()
        self._config["nonAsciiPlaceholder"] = "."

    def process(self, action, _bytes):
        self._mergeConfig(action)
        self._cnvrtr._nonAsciiPlaceholder = self._config["nonAsciiPlaceholder"]
        return self._cnvrtr.hexToAsciiString(_bytes.hex())


from ntrprtr.action.ActionBase import ActionBase

class EndianessAction(ActionBase):
    def __init__(self):
        super().__init__()
        self._config["endianess"] = "little"
      
    def process(self, action, _bytes):
        self._mergeConfig(action)
        result = ""
        endianess = self._config["endianess"]
        if(endianess == "big"):
            result = _bytes.hex(" ")
        elif(endianess == "little"):
            result = self._cnvrtr.toLittleEndian(_bytes.hex(" "))
        result = " ".join(result[i:i+2] for i in range(0, len(result), 2))
        return result.upper()


from cnvrtr.Converter import Converter

class ActionBase():
    def __init__(self) -> None:
        self._cnvrtr = Converter()
        self._config = {}

    def _mergeConfig(self, action):
        self._config |= action



from ntrprtr.action.ActionBase import ActionBase

class EqualsAction(ActionBase):
    def __init__(self):
        super().__init__()
        self._config["endianess"] = "big"
        self._config["noMatch"] = "No match found!"

    def process(self, action, _bytes):
        self._mergeConfig(action)
       
        b = ""
        result = self._config["noMatch"]
        endianess = self._config["endianess"]

        if(endianess == "big"):
            b = _bytes.hex()
        elif(endianess == "little"):
            b = self._cnvrtr.toLittleEndian(_bytes.hex(" "))

        for i in range(0, len(self._config["cmp"])):
            if(b == self._config["cmp"][i]["value"].lower()):
                result = self._config["cmp"][i]["description"]
        return result